package org.example;

import net.jqwik.api.*;
import net.jqwik.api.constraints.IntRange;
import org.junit.jupiter.api.Assertions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class Main {
    public static void main(String[] args) {
        System.out.println("Hello world!");
    }

    @Property
    void reverseList(@ForAll List inputList) {
        List reversed = reverse(inputList);
        Collections.reverse(inputList); // Reverse the input using a known method

        assertEquals(inputList, reversed);
    }

    // A simple function to reverse a list
    public static List reverse(List list) {
        List reversedList = new ArrayList<>();
        for (int i = list.size() - 1; i >= 0; i--) {
            reversedList.add(list.get(i));
        }
        return reversedList;
    }

    @Property
    void testSquareOfPositiveNumber(@ForAll("positiveIntegers") int num) {
        assertTrue(num > 0);
        int result = square(num);
        assertEquals(square(num), result);
    }

    int square(int num) {
        return num * num;
    }

    @Provide("positiveIntegers")
    Arbitrary<Integer> positiveIntegers() {
        return Arbitraries.integers().greaterOrEqual(1); // Generate positive integers
    }

    @Property
    void testStringLength(@ForAll("customStrings") String text) {
        System.out.println(text);
        // Property assertions go here
    }

    @Provide
    Arbitrary customStrings() {
        return Arbitraries.strings().withCharRange('a', 'z').ofLength(5); // Generate 5-character strings
    }

    @Property(shrinking = ShrinkingMode.FULL)
    void testSumIsAlwaysGreaterThanInput(
            @ForAll @IntRange(min = -100, max = 100) int a,
            @ForAll @IntRange(min = -100, max = 100) int b
    ) {
        assertTrue(a + b > a);
    }

}